#define _GNU_SOURCE
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <termios.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <gnome-keyring.h>

const char *itemtype_string(GnomeKeyringItemInfo *item)
{
    if (!item)
        return "None (NULL)";

    switch (gnome_keyring_item_info_get_type(item)) {
    case GNOME_KEYRING_ITEM_GENERIC_SECRET:
        return "GNOME_KEYRING_ITEM_GENERIC_SECRET (Generic secret)";
    case GNOME_KEYRING_ITEM_NETWORK_PASSWORD:
        return "GNOME_KEYRING_ITEM_NETWORK_PASSWORD (Network password)";
    case GNOME_KEYRING_ITEM_NOTE:
        return "GNOME_KEYRING_ITEM_NOTE (Note)";
    case GNOME_KEYRING_ITEM_CHAINED_KEYRING_PASSWORD:
        return "GNOME_KEYRING_ITEM_CHAINED_KEYRING_PASSWORD (Keyring password)";
    case GNOME_KEYRING_ITEM_ENCRYPTION_KEY_PASSWORD:
        return "GNOME_KEYRING_ITEM_ENCRYPTION_KEY_PASSWORD (Password for encryption key)";
    default:
        return "Other";
    }
}

int main(void)
{
    GnomeKeyringResult result;
    GList *list = NULL, *curr;

    /* Unlock the keyring. */
    result = gnome_keyring_unlock_sync("login", NULL);
    if (result != GNOME_KEYRING_RESULT_ALREADY_UNLOCKED &&
        result != GNOME_KEYRING_RESULT_OK) {
        fprintf(stderr, "Error unlocking 'login' keyring: %s.\n", gnome_keyring_result_to_message(result));
        return EXIT_FAILURE;
    }

    fprintf(stderr, "'login' keyring unlocked.\n");
    fflush(stderr);

    /* List items. */
    result = gnome_keyring_list_item_ids_sync("login", &list);
    if (result != GNOME_KEYRING_RESULT_OK) {
        fprintf(stderr, "Cannot list keyring items: %s.\n", gnome_keyring_result_to_message(result));
        return EXIT_FAILURE;
    }

    for (curr = list; curr != NULL; curr = curr->next) {
        GnomeKeyringItemInfo *info = NULL;
        GnomeKeyringAttributeList *attrlist = NULL;
        char *name;
        unsigned int i;

        result = gnome_keyring_item_get_info_full_sync("login", GPOINTER_TO_UINT(curr->data), 0, &info);
        if (result != GNOME_KEYRING_RESULT_OK) {
            fprintf(stderr, "Cannot get item %u information.\n", GPOINTER_TO_UINT(curr->data));
            continue;
        }
        gnome_keyring_item_get_attributes_sync("login", GPOINTER_TO_UINT(curr->data), &attrlist);

        name = gnome_keyring_item_info_get_display_name(info);

        printf("Item %u:\n", GPOINTER_TO_UINT(curr->data));
        if (name)
        printf("\tName: %s\n", name);
        printf("\tType: %s\n", itemtype_string(info));

        if (attrlist) for (i = 0; i < attrlist->len; i++) {
            GnomeKeyringAttribute *attr = &g_array_index(attrlist, GnomeKeyringAttribute, i);
            printf("\tAttribute '%s': (%d)\n", attr->name, i);
            if (attr->type == GNOME_KEYRING_ATTRIBUTE_TYPE_STRING)
                printf("\t\t'%s'\n", gnome_keyring_attribute_get_string(attr));
            else
            if (attr->type == GNOME_KEYRING_ATTRIBUTE_TYPE_UINT32)
                printf("\t\t%u\n", gnome_keyring_attribute_get_uint32(attr));
            else
                printf("\t\tUnknown value\n");
        }

        gnome_keyring_attribute_list_free(attrlist);
        free(name);
        gnome_keyring_item_info_free(info);
    }

    /* Free item list. */
    g_list_free(list);

    /* Success. */
    return EXIT_SUCCESS;
}
